@@ -378,13 +378,15 @@ Rcpp::List PF_smooth(
     const arma::vec &tstart,
     const arma::vec &tstop,
     const arma::colvec &a_0,
+    const arma::mat &R,
+    const arma::mat &L,
+    const arma::vec &m,
     arma::mat &Q_0,
     arma::mat &Q,
     const arma::mat Q_tilde,
     const Rcpp::List &risk_obj,
     const arma::mat &F,
     const int n_max,
-    const int order,
     const int n_threads,
     const int N_fw_n_bw,
     const int N_smooth,
@@ -396,7 +398,7 @@ Rcpp::List PF_smooth(
     const std::string model){
   const arma::ivec is_event_in_bin = Rcpp::as<arma::ivec>(risk_obj["is_event_in"]);
 
-  PF_data data(
+  random_walk<PF_data> data(
       n_fixed_terms_in_state_vec,
       X,
       fixed_terms,
@@ -404,12 +406,14 @@ Rcpp::List PF_smooth(
       tstop,
       is_event_in_bin,
       a_0,
+      R,
+      L,
+      m,
       Q_0,
       Q,
       risk_obj,
       F,
       n_max,
-      order,
       n_threads,
       Q_tilde,
       N_fw_n_bw,
@@ -421,10 +425,10 @@ Rcpp::List PF_smooth(
   Rcpp::List ans;
 
   if(model == "logit"){
-    ans = PF_smooth_dens<binary>::compute(data, smoother, method);
+    ans = PF_smooth_dens<logistic_dens>::compute(data, smoother, method);
 
   } else if (model == "exponential"){
-    ans = PF_smooth_dens<exponential>::compute(data, smoother, method);
+    ans = PF_smooth_dens<exponential_dens>::compute(data, smoother, method);
 
   } else {
     std::stringstream stream;
